/*
** Copyright (C) 2001-2018 by Carnegie Mellon University.
**
** @OPENSOURCE_LICENSE_START@
** See license information in ../../LICENSE.txt
** @OPENSOURCE_LICENSE_END@
*/
#ifndef _SILK_IPSET_H
#define _SILK_IPSET_H
#ifdef __cplusplus
extern "C" {
#endif

#include "silk-ipset-config.h"

#include <stdio.h>
#ifdef    SK_HAVE_SYS_TYPES_H
#  include <sys/types.h>
#endif
#ifdef    SK_HAVE_SYS_STAT_H
#  include <sys/stat.h>
#endif
#ifdef    SK_STDC_HEADERS
#  include <stdlib.h>
#  include <stddef.h>
#else
#  ifdef  SK_HAVE_STDLIB_H
#    include <stdlib.h>
#  endif
#  ifdef  SK_HAVE_MALLOC_H
#    include <malloc.h>
#  endif
#endif
#ifdef    SK_HAVE_STRING_H
#  if     !defined SK_STDC_HEADERS && defined SK_HAVE_MEMORY_H
#    include <memory.h>
#  endif
#  include <string.h>
#endif
#ifdef    SK_HAVE_STRINGS_H
#  include <strings.h>
#endif
#if defined(__cplusplus) && !defined(__STDC_FORMAT_MACROS) && !defined(PRIu32)
#define __STDC_FORMAT_MACROS
#endif
#ifdef    SK_HAVE_INTTYPES_H
#  include <inttypes.h>
#endif
#ifdef    SK_HAVE_STDINT_H
#  include <stdint.h>
#endif
#ifdef    SK_HAVE_UNISTD_H
#  include <unistd.h>
#endif

#ifdef    SK_HAVE_ASSERT_H
#  include <assert.h>
#endif
#ifdef    SK_HAVE_CTYPE_H
#  include <ctype.h>
#endif
#ifdef    SK_HAVE_ERRNO_H
#  include <errno.h>
#endif
#ifdef    SK_HAVE_FCNTL_H
#  include <fcntl.h>
#endif
#ifdef    SK_HAVE_FLOAT_H
#  include <float.h>
#endif
#ifdef    SK_HAVE_LIMITS_H
#  include <limits.h>
#endif
#ifdef    SK_HAVE_MATH_H
#  include <math.h>
#endif
#ifdef    SK_HAVE_STDARG_H
#  include <stdarg.h>
#endif
#ifdef    SK_HAVE_ARPA_INET_H
#  include <arpa/inet.h>
#endif

#ifdef    SK_TIME_WITH_SYS_TIME
#  include <sys/time.h>
#  include <time.h>
#else
#  ifdef  SK_HAVE_SYS_TIME_H
#    include <sys/time.h>
#  else
#    include <time.h>
#  endif
#endif

#ifdef    SK_HAVE_SYS_MMAN_H
#  include <sys/mman.h>
#endif
#ifdef    SK_HAVE_SYS_WAIT_H
#  include <sys/wait.h>
#endif

/* support for the SILK_CLOBBER environment variable; undefine to disable */
#define SILK_CLOBBER_ENVAR  "SILK_CLOBBER"

/* Add appropriate suffix to constants */
#if !defined(INT32_C)
/* Assume we either get them all or get none of them. */
#  define  INT8_C(v)      (v)
#  define  INT16_C(v)     (v)
#  define  INT32_C(v)     (v)

#  define UINT8_C(v)      (v ## U)
#  define UINT16_C(v)     (v ## U)
#  define UINT32_C(v)     (v ## U)

#  if (SK_SIZEOF_LONG >= 8)
#    define  INT64_C(v)   (v ## L)
#    define UINT64_C(v)   (v ## UL)
#  elif (SK_SIZEOF_UNSIGNED_LONG_LONG >= 8)
#    define  INT64_C(v)   (v ## LL)
#    define UINT64_C(v)   (v ## ULL)
#  endif
#endif


/* Maxima */
#if !defined(INT8_MAX)
#  define  INT8_MAX          127
#endif
#if !defined(UINT8_MAX)
#  define UINT8_MAX          255U
#endif
#if !defined(INT16_MAX)
#  define  INT16_MAX       32767
#endif
#if !defined(UINT16_MAX)
#  define UINT16_MAX       65535U
#endif
#if !defined(INT32_MAX)
#  define  INT32_MAX   2147483647
#endif
#if !defined(UINT32_MAX)
#  define UINT32_MAX   4294967295U
#endif
#if !defined(INT64_MAX)
#  define  INT64_MAX    INT64_C(9223372036854775807)
#endif
#if !defined(UINT64_MAX)
#  define UINT64_MAX   UINT64_C(18446744073709551615)
#endif

#if !defined(ULLONG_MAX)
#  if (SK_SIZEOF_UNSIGNED_LONG_LONG == 8)
#    define ULLONG_MAX UINT64_MAX
#  endif
#endif
#if !defined(SIZE_MAX)
#  if (SK_SIZEOF_SIZE_T >= 8)
#    define SIZE_MAX   UINT64_MAX
#  else
#    define SIZE_MAX   UINT32_MAX
#  endif
#endif
#if !defined(SSIZE_MAX)
#  if (SK_SIZEOF_SSIZE_T >= 8)
#    define SSIZE_MAX  INT64_MAX
#  else
#    define SSIZE_MAX  INT32_MAX
#  endif
#endif

/* PRI* macros for printing */
#if !defined(PRIu32)
/* Assume we either get them all or get none of them. */
#  define PRId32 "d"
#  define PRIi32 "i"
#  define PRIo32 "o"
#  define PRIu32 "u"
#  define PRIx32 "x"
#  define PRIX32 "X"

#  define PRId16 PRId32
#  define PRIi16 PRIi32
#  define PRIo16 PRIo32
#  define PRIu16 PRIu32
#  define PRIx16 PRIx32
#  define PRIX16 PRIX32

#  define PRId8  PRId32
#  define PRIi8  PRIi32
#  define PRIo8  PRIo32
#  define PRIu8  PRIu32
#  define PRIx8  PRIx32
#  define PRIX8  PRIX32
#endif /* !defined(PRIU32) */
#if !defined(PRIu64)
#  if (SK_SIZEOF_LONG >= 8)
#    define PRId64 "l" PRId32
#    define PRIi64 "l" PRIi32
#    define PRIo64 "l" PRIo32
#    define PRIu64 "l" PRIu32
#    define PRIx64 "l" PRIx32
#    define PRIX64 "l" PRIX32
#  else
#    define PRId64 "ll" PRId32
#    define PRIi64 "ll" PRIi32
#    define PRIo64 "ll" PRIo32
#    define PRIu64 "ll" PRIu32
#    define PRIx64 "ll" PRIx32
#    define PRIX64 "ll" PRIX32
#  endif
#endif /* !defined(PRIu64) */
#if !defined(PRIuMAX)
#  define PRIdMAX PRId64
#  define PRIiMAX PRIi64
#  define PRIoMAX PRIo64
#  define PRIuMAX PRIu64
#  define PRIxMAX PRIx64
#  define PRIXMAX PRIX64
#endif

/* figure out how to print size_t and ssize_t:
 * printf("%" PRIuZ "\n", SK_CAST_SIZE_T(sizeof(x))); */
#if defined(SK_HAVE_PRINTF_Z_FORMAT)
#  define SK_PRIdZ "zd"
#  define SK_PRIiZ "zi"
#  define SK_PRIoZ "zo"
#  define SK_PRIuZ "zu"
#  define SK_PRIxZ "zx"
#  define SK_PRIXZ "zX"
#  define SK_CAST_SIZE_T(scs_x)  (scs_x)
#  define SK_CAST_SSIZE_T(scs_x) (scs_x)
#elif (SK_SIZEOF_SIZE_T == SK_SIZEOF_LONG)
#  define SK_PRIdZ "ld"
#  define SK_PRIiZ "li"
#  define SK_PRIoZ "lo"
#  define SK_PRIuZ "lu"
#  define SK_PRIxZ "lx"
#  define SK_PRIXZ "lX"
#  define SK_CAST_SIZE_T(scs_x)  ((unsigned long)scs_x)
#  define SK_CAST_SSIZE_T(scs_x) ((long)scs_x)
#elif (SK_SIZEOF_SIZE_T == SK_SIZEOF_UNSIGNED_LONG_LONG)
#  define SK_PRIdZ "lld"
#  define SK_PRIiZ "lli"
#  define SK_PRIoZ "llo"
#  define SK_PRIuZ "llu"
#  define SK_PRIxZ "llx"
#  define SK_PRIXZ "llX"
#  define SK_CAST_SIZE_T(scs_x)  ((unsigned long long)scs_x)
#  define SK_CAST_SSIZE_T(scs_x) ((long long)scs_x)
#else
#  define SK_PRIdZ "d"
#  define SK_PRIiZ "i"
#  define SK_PRIoZ "o"
#  define SK_PRIuZ "u"
#  define SK_PRIxZ "x"
#  define SK_PRIXZ "X"
#  define SK_CAST_SIZE_T(scs_x)  ((unsigned int)scs_x)
#  define SK_CAST_SSIZE_T(scs_x) ((int)scs_x)
#endif


/* Functions and types from C99 that we require */
#ifndef SK_HAVE_INTMAX_T
#  define intmax_t sk_intmax_t
#endif
#ifndef SK_HAVE_IMAXDIV_T
#  define imaxdiv_t sk_imaxdiv_t
#endif
#ifndef SK_HAVE_IMAXDIV
#  define imaxdiv sk_imaxdiv
#endif
#ifndef SK_HAVE_MEMCCPY
#  define memccpy sk_memccpy
#endif
#ifndef SK_HAVE_SETENV
#  define setenv sk_setenv
#endif
#ifndef SK_HAVE_STRSEP
#  define strsep sk_strsep
#endif

/*
 *    Convert a three part version number MAJOR, MINOR, PATCH to an
 *    integer.
 */
#define SK_VERS3_TO_NUMBER(m_vtn1, m_vtn2, m_vtn3)              \
    (((((m_vtn1) * 1000) + (m_vtn2)) * 1000) + (m_vtn3))

/*
 *    Compute an integer for the current gcc version.
 */
#ifndef __GNUC__
#define SK_GNUC_VERSION 0
#else
#define SK_GNUC_VERSION                                                 \
    SK_VERS3_TO_NUMBER(__GNUC__, __GNUC_MINOR__, __GNUC_PATCHLEVEL__)
#endif

/* Wrap UNUSED() around unused parameters to silence gcc -W */
#ifndef __GNUC__
#define UNUSED(var) /*@unused@*/ var
#define NORETURN
#else
#define UNUSED(var) /*@unused@*/ var __attribute__((__unused__))
#define NORETURN  __attribute__((__noreturn__))
#endif

/*  Allow gcc to check parameters to printf-style functions and
 *  to printf-style function-pointer typedefs */
#ifndef __GNUC__
#define SK_CHECK_PRINTF(fmt_pos, argv1)
#define SK_CHECK_TYPEDEF_PRINTF(fmt_pos, argv1_pos)
#else
#define SK_CHECK_PRINTF(fmt_pos, argv1_pos)                     \
    __attribute__((format (printf, fmt_pos, argv1_pos)))
#ifndef SK_HAVE_COMP_ATTRIBUTE_PRINTF_TYPEDEF
#define SK_CHECK_TYPEDEF_PRINTF(fmt_pos, argv1_pos)
#else
#define SK_CHECK_TYPEDEF_PRINTF(fmt_pos, argv1_pos)             \
    SK_CHECK_PRINTF(fmt_pos, argv1_pos)
#endif
#endif

/*
 *  Macros to check whether the compiler supports #pragma statements
 *  for pushing and popping the state of diagnostic messages.
 */
#if defined(__clang__)
/*  Assume all versions of clang support push and pop */

#  define SK_DIAG_STR(ds_str)  #ds_str
#  define SK_DIAGNOSTIC_IGNORE_PUSH(dip_str)                    \
    _Pragma("clang diagnostic push")                            \
    _Pragma(SK_DIAG_STR(clang diagnostic ignored dip_str))
#  define SK_DIAGNOSTIC_IGNORE_POP(dip_str)     \
    _Pragma("clang diagnostic pop")

/* clang complains about non-literal format strings for functions that
 * take varargs. Disable the warnings for these functions */
#  define SK_DIAGNOSTIC_FORMAT_NONLITERAL_PUSH          \
    SK_DIAGNOSTIC_IGNORE_PUSH("-Wformat-nonliteral")
#  define SK_DIAGNOSTIC_FORMAT_NONLITERAL_POP           \
    SK_DIAGNOSTIC_IGNORE_POP("-Wformat-nonliteral")     \

#  define SK_GCC_DEPRECATED     __attribute__((__deprecated__))

#elif defined(__GNUC__)

/*
 *  '#pragma GCC diagnostic push/pop' requires gcc 4.6 or later.
 *  '#pragma GCC diagnostic warning' requires gcc 4.2.4 or later.
 *  '#pragma GCC diagnostic ignore' requires gcc 4.2 or later.
 */
#if SK_GNUC_VERSION >= SK_VERS3_TO_NUMBER(4, 6, 0)

#  define SK_DIAG_STR(ds_str)  #ds_str
#  define SK_DIAGNOSTIC_IGNORE_PUSH(dip_str)                    \
    _Pragma("GCC diagnostic push")                              \
    _Pragma(SK_DIAG_STR(GCC diagnostic ignored dip_str))
#  define SK_DIAGNOSTIC_IGNORE_POP(dip_str)     \
    _Pragma("GCC diagnostic pop")

#elif SK_GNUC_VERSION >= SK_VERS3_TO_NUMBER(4, 2, 0)

#  define SK_DIAG_STR(ds_str)  #ds_str
#  define SK_DIAGNOSTIC_IGNORE_PUSH(dip_str)                    \
    _Pragma(SK_DIAG_STR(GCC diagnostic ignored dip_str))

#if SK_GNUC_VERSION >= SK_VERS3_TO_NUMBER(4, 2, 4)
#  define SK_DIAGNOSTIC_IGNORE_POP(dip_str)                     \
    _Pragma(SK_DIAG_STR(GCC diagnostic warning dip_str))
#endif
#endif  /* GCC version for push/pop, warning, ignore */

#if SK_GNUC_VERSION >= SK_VERS3_TO_NUMBER(3, 1, 0)
#  define SK_GCC_DEPRECATED     __attribute__((__deprecated__))
#endif

#endif  /* Compiler: clang, GCC */

#ifndef SK_DIAGNOSTIC_IGNORE_PUSH
#  define SK_DIAGNOSTIC_IGNORE_PUSH(dip_str)
#endif
#ifndef SK_DIAGNOSTIC_IGNORE_POP
#  define SK_DIAGNOSTIC_IGNORE_POP(dip_str)
#endif
#ifndef SK_DIAGNOSTIC_FORMAT_NONLITERAL_PUSH
#  define SK_DIAGNOSTIC_FORMAT_NONLITERAL_PUSH
#endif
#ifndef SK_DIAGNOSTIC_FORMAT_NONLITERAL_POP
#  define SK_DIAGNOSTIC_FORMAT_NONLITERAL_POP
#endif
#ifndef SK_GCC_DEPRECATED
#  define SK_GCC_DEPRECATED
#endif


/*  Alternate approach to marking unused parameters */
#define SK_UNUSED_PARAM(unused_param)   (void)(unused_param)


/*
 *  Create a variable for magic RCS variables: Define 'var' to be the
 *  string in 'id'.  Can be used in header, keep 'var's unique.  Use:
 *  RCSIDENTVAR(myheader, "$Magic RCS Var$");
 */
#ifndef SK_HAVE_COMP_ATTRIBUTE_USED
#define RCSIDENTVAR(var, id) \
    static const char /*@observer@*/ UNUSED(* var) = (id)
#else
/*
 *  Have RCSIDENTVAR() mark the variables as "used" so GCC doesn't
 *  strip them.
 */
#define RCSIDENTVAR(var, id) \
    static const char /*@observer@*/ /*@unused@*/ * var __attribute__((__used__)) = (id)
#endif /* SK_HAVE_COMP_ATTRIBUTE_USED */

/*
 *  As above, except for C files since 'var' can be fixed.
 */
#define RCSIDENT(id) RCSIDENTVAR(_rcsID, (id))


RCSIDENTVAR(rcsID_SILK_H, "$SiLK: silk-ipset.h e85db1eebd9f 2018-12-13 18:33:20Z mthomas $");


/*
 *    Functions declared in utils.h, defined in sku-app.c, that print
 *    messages to the stderr.
 */
void
skAppPrintAbortMsg(
    const char         *func_name,
    const char         *file_name,
    int                 line_number);
void
skAppPrintBadCaseMsg(
    const char         *func_name,
    const char         *file_name,
    int                 line_number,
    int64_t             value,
    const char         *value_expr);

/* include abort() in macro definition so compiler knows function exits */
#ifdef SK_HAVE_C99___FUNC__
#define skAbort()                                               \
    do {                                                        \
        skAppPrintAbortMsg(__func__, __FILE__, __LINE__);       \
        abort();                                                \
    } while(0)
#define skAbortBadCase(abc_expr)                                \
    do {                                                        \
        skAppPrintBadCaseMsg(__func__, __FILE__, __LINE__,      \
                             (int64_t)(abc_expr), #abc_expr);   \
        abort();                                                \
    } while(0)
#else
#define skAbort()                                       \
    do {                                                \
        skAppPrintAbortMsg(NULL, __FILE__, __LINE__);   \
        abort();                                        \
    } while(0)
#define skAbortBadCase(abc_expr)                                \
    do {                                                        \
        skAppPrintBadCaseMsg(NULL, __FILE__, __LINE__,          \
                             (int64_t)(abc_expr), #abc_expr);   \
        abort();                                                \
    } while(0)
#endif


/* Bit-swapping macros for changing endianness */
#if  SK_LITTLE_ENDIAN
#  define  BSWAP16(a)  (ntohs(a))
#  define  BSWAP32(a)  (ntohl(a))
#else
#  define BSWAP16(a) ((((uint16_t)(a)) >> 8) |    \
                      (((uint16_t)(a)) << 8))
#  define BSWAP32(a) (((((uint32_t)(a)) & 0x000000FF) << 24) |    \
                      ((((uint32_t)(a)) & 0x0000FF00) << 8)  |    \
                      ((((uint32_t)(a)) & 0x00FF0000) >> 8)  |    \
                      ((((uint32_t)(a)) >> 24) & 0x000000FF))
#endif /* SK_LITTLE_ENDIAN */
#define BSWAP64(a)                                              \
    ((((uint64_t)BSWAP32((uint32_t)((a) & 0xffffffff))) << 32)  \
     | BSWAP32((uint32_t)((a) >> 32)))


/* provide a network-to-host swapper for 64bit values */
#if   SK_LITTLE_ENDIAN
#  define ntoh64(a)  BSWAP64(a)
#else
#  define ntoh64(a)  (a)
#endif
#define hton64(a)  ntoh64(a)


/*
 *    The IP address structure.  Most code should use skipaddr_t
 *    instead of skIPUnion_t.  Users should not deference directly, use
 *    the macros specified in skipaddr.h to get/set the value.
 */
typedef union skipunion_un {
    uint32_t    ipu_ipv4;
#if SK_ENABLE_IPV6
    uint8_t     ipu_ipv6[16];
#endif
} skIPUnion_t;

/*
 *    An IP address structure that knows the version of IP address it
 *    contains.  Do not deference directly, instead use the macros
 *    specified in skipaddr.h to get/set the value
 */
typedef struct skipaddr_st {
    skIPUnion_t ip_ip;
#if SK_ENABLE_IPV6
    unsigned    ip_is_v6 :1;
#endif
} skipaddr_t;

/**
 *    Flags that determine the string representation of an IP as
 *    returned by skipaddrString() and other functions declared in
 *    utils.h
 */
typedef enum {
    /**
     *  Canonical format: dotted quad for IPv4 and colon-separated
     *  hexadecimal for IPv6.  This format calls the inet_ntop(3)
     *  function which uses a mixture of IPv4 and IPV6 addresss for
     *  IPv6 addresses that occur in the ::ffff:0:0/96 and ::/96
     *  netblocks (except ::/127), such as ::192.0.2.1 and
     *  ::ffff:192.0.2.1.
     *
     *  The maximum length of this string is 15 for IPv4, 39 for IPv6,
     *  18 for IPv4/CIDR, 43 for IPv6/CIDR.
     *
     *  May be used with SKIPADDR_ZEROPAD and either SKIPADDR_MAP_V4
     *  or SKIPADDR_UNMAP_V6.
     */
    SKIPADDR_CANONICAL = 0,
    /**
     *  Value as an integer number, printed as decimal.
     *
     *  The maximum length of this string is 10 for IPv4, 39 for IPv6,
     *  13 for IPv4/CIDR, 43 for IPv6/CIDR.
     *
     *  May be used with SKIPADDR_ZEROPAD and either SKIPADDR_MAP_V4
     *  or SKIPADDR_UNMAP_V6.
     */
    SKIPADDR_DECIMAL = 1u,
    /**
     *  Value as an integer number, printed as hexadecimal.
     *
     *  The maximum length of this string is 8 for IPv4, 32 for IPv6,
     *  11 for IPv4/CIDR, 36 for IPv6/CIDR.
     *
     *  May be used with SKIPADDR_ZEROPAD and either
     *  SKIPADDR_MAP_V4 or SKIPADDR_UNMAP_V6.
     */
    SKIPADDR_HEXADECIMAL = 2u,
    /**
     *  Print IPv4 addresses in the canonical form.  Print IPv6 in the
     *  canonical form but use only hexadecimal numbers and never a
     *  mixture of IPv4 and IPv6.  That is, prints ::c000:201 instead
     *  of ::192.0.2.1.
     *
     *  May be used with SKIPADDR_ZEROPAD and either SKIPADDR_MAP_V4
     *  or SKIPADDR_UNMAP_V6.
     *
     *  Since SiLK 3.17.0.
     */
    SKIPADDR_NO_MIXED = 3u,
    /**
     *  Apply any other formatting, and then pad with zeros to expand
     *  the string to the maximum width: Each IPv4 octet has three
     *  characters, each IPv6 hexadectet has 4 characters, decimal and
     *  hexadecimal numbers have leading zeros.  Useful for string
     *  comparisons.
     *
     *  May be used with any other skipaddr_flags_t.
     */
    SKIPADDR_ZEROPAD = (1u << 7),
    /**
     *  Map each IPv4 address into the IPv6 ::ffff:0:0/96 netblock and
     *  then apply other formatting.  Has no effect on IPv6 addresses.
     *
     *  May be used with any other skipaddr_flags_t except
     *  SKIPADDR_UNMAP_V6.
     *
     *  Since SiLK 3.17.0.
     */
    SKIPADDR_MAP_V4 = (1u << 8),
    /**
     *  For each IPv4-mapped IPv6 address (i.e., IPv6 addresses in the
     *  ::ffff:0:0/96 netblock), convert the address to IPv4 and then
     *  apply other formatting.  Has no effect on IPv4 addresses or on
     *  IPv6 addresses outside of the ::ffff:0:0/96 netblock.
     *
     *  May be used with any other skipaddr_flags_t except
     *  SKIPADDR_MAP_V4 and SKIPADDR_FORCE_IPV6.
     *
     *  Since SiLK 3.17.0.
     */
    SKIPADDR_UNMAP_V6 = (1u << 9),
    /**
     *  Map each IPv4 address into the IPv6 ::ffff:0:0/96 netblock and
     *  then apply the SKIPADDR_NO_MIXED format.
     *
     *  May be used with SKIPADDR_ZEROPAD.
     */
    SKIPADDR_FORCE_IPV6 = (SKIPADDR_MAP_V4 | SKIPADDR_NO_MIXED)
} skipaddr_flags_t;

/**
 *    Minimum required size of character buffer that holds the printed
 *    representation of an IP address.  skipaddrString() expects (and
 *    assumes) a buffer of at least this size.
 *
 *    The length is taken from INET6_ADDRSTRLEN used by inet_ntop(),
 *    which can return "0000:0000:0000:0000:0000:ffff:000.000.000.000"
 */
#define SKIPADDR_STRLEN     46
#define SK_NUM2DOT_STRLEN   SKIPADDR_STRLEN

/**
 *    Minimum required size of character buffer that holds the printed
 *    representation of an IP address, a slash ('/'), and a CIDR
 *    prefix (netblock) designation.  skipaddrCidrString() expects
 *    (and assumes) a buffer of at least this size.
 *
 *    Since SiLK 3.18.0.
 */
#define SKIPADDR_CIDR_STRLEN    (SKIPADDR_STRLEN + 4)

/*
 *    How to handle IPv6 Flow records.
 */
typedef enum sk_ipv6policy_en {
    /* completely ignore IPv6 flows */
    SK_IPV6POLICY_IGNORE = -2,
    /* convert IPv6 flows to IPv4 if possible, else ignore */
    SK_IPV6POLICY_ASV4 = -1,
    /* mix IPv4 and IPv6 flows in the result--this is the default */
    SK_IPV6POLICY_MIX = 0,
    /* force IPv4 flows to be converted to IPv6 */
    SK_IPV6POLICY_FORCE = 1,
    /* only return IPv6 flows that were marked as IPv6 */
    SK_IPV6POLICY_ONLY = 2
} sk_ipv6policy_t;

/*
 *    A special structure of IP Addresses.  It is defined in utils.h
 */
typedef struct skIPWildcard_st skIPWildcard_t;


/* *****  STREAM / FILE FORMATS  ********************************** */

/*
 *    Interface to a file containing SiLK Data---flow records or
 *    IPsets, etc---is an skstream_t.  See skstream.h.
 */
typedef struct skstream_st skstream_t;

/*
 *    Type to hold the ID of the various SiLK file formats.  The
 *    format IDs begin with FT_ and are listed in silk_files.h.
 */
typedef uint8_t  sk_file_format_t;
typedef sk_file_format_t fileFormat_t       SK_GCC_DEPRECATED;

/*
 *    The value for an invalid or unrecognized file format.
 *
 *    Since SiLK 3.13.0.
 */
#define SK_INVALID_FILE_FORMAT      ((sk_file_format_t)0xFF)

/*
 *    The strlen() of the names of file formats will be this size or
 *    less.
 */
#define SK_MAX_STRLEN_FILE_FORMAT   32

/*
 *    A version of the file format.
 */
typedef uint8_t  sk_file_version_t;
typedef sk_file_version_t fileVersion_t     SK_GCC_DEPRECATED;

/*
 *    Value meaning that any file version is valid
 */
#define SK_RECORD_VERSION_ANY       ((sk_file_version_t)0xFF)

/*
 *    The compression method used to write the data section of a file.
 *    The known compression methods are listed in silk_files.h.
 */
typedef uint8_t sk_compmethod_t;

/*
 *    The value for an invalid or unrecognized compression method
 */
#define SK_INVALID_COMPMETHOD       ((sk_compmethod_t)0xFF)

/*
 *    Values that specify how a stream/file is to be opened.
 */
typedef enum {
    SK_IO_READ = 1,
    SK_IO_WRITE = 2
} skstream_mode_t;

/*
 *    What type of content the stream contains
 */
typedef enum {
    /* stream contains line-oriented text */
    SK_CONTENT_TEXT = (1 << 0),
    /* stream contains a SiLK file header and data (non-Flow data) */
    SK_CONTENT_SILK = (1 << 2)
} skcontent_t;


/*
 *    Bitmap of integers.  It is defined in utils.h.
 */
typedef struct sk_bitmap_st sk_bitmap_t;


/*
 *    An enumeration type for endianess.
 */
typedef enum silk_endian_en {
    SILK_ENDIAN_BIG,
    SILK_ENDIAN_LITTLE,
    SILK_ENDIAN_NATIVE,
    SILK_ENDIAN_ANY
} silk_endian_t;


/*
 *    The status of an iterator.
 */
typedef enum skIteratorStatus_en {
    /* More entries */
    SK_ITERATOR_OK=0,
    /* No more entries */
    SK_ITERATOR_NO_MORE_ENTRIES
} skIteratorStatus_t;

/*
 *    The type of message functions.  These should use the same
 *    semantics as printf.
 */
typedef int (*sk_msg_fn_t)(const char *, ...);

/*
 *    The type of message functions with the arguments expanded to a
 *    variable argument list.
 */
typedef int (*sk_msg_vargs_fn_t)(const char *, va_list);


#include "skfile.h"
#include "skipaddr.h"
#include "skheader.h"
#include "skipset.h"
#include "skiptree.h"
#include "skstream.h"
#include "skutils.h"

#ifdef __cplusplus
}
#endif
#endif /* _SILK_IPSET_H */

/*
** Local Variables:
** mode:c
** indent-tabs-mode:nil
** c-basic-offset:4
** End:
*/
